class Headers:
    authorization = "Authorization"
    x_enabled_repo = "Enabled-Repo"
    x_auth_userid = "X-Auth-Userid"
    x_auth_username = "X-Auth-Username"
    x_auth_email = "X-Auth-Email"
    x_project_id = "X-Project-Id"
    x_request_id = "X-Request-Id"
    x_repo_id = "X-Repo-Id"
    x_branch_name = "X-Branch-Name"
    x_default_repo_flag = "X_Default_Repo_Flag"
    x_product_id = "X-Product-Id"
    x_tenant_id = "X-Auth-TenantId"
    x_auth_realm_name = "X-Auth-RealmName"
    access_control_allow_origin = "Access-Control-Allow-Origin"
    access_control_allow_methods = "Access-Control-Allow-Methods"
    access_control_allow_headers = "Access-Control-Allow-Headers"
    x_cli_script = "X-Cli-Script"
