# -*- coding: utf-8 -*-

"""Combine dataframes."""


class CombineDFs(object):
    """Combine two DataFrames."""

    @property
    def __name__(self):
        """Instance name is the same as the class name."""
        return self.__class__.__name__
