class Url:
    # lens_manage_console_url = "http://lens-manageconsole:80/lens-manageconsole/insight-hub/notification-management/notification-add"
    notification_url = "http://notification-service/notification/v1/notification-add"
    WHOAMI_URL = "http://user-management:8094/usermgm/user/whomi"
    LINE_MANAGER_URL = "http://user-management:8094/usermgm/accessRequest/getManagerDetailsByUsersBankId/bankId="
