# -*- coding: utf-8 -*-
class GenericConstants:
    shared_volumes = "shared_volume"
    slash = "/"
    project_data_dir = "{0}/{1}-Data"
    snapshot_data_fir= "{0}/{1}-Snapshot/{2}"
    default = "input"
    all_snapshot_data= "{0}/{1}-Snapshot"
